#ifndef MISC_H
#define MISC_H

#include <qvaluelist.h>
#include <qdialog.h>
#include <qcombobox.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qcstring.h>
#include <qdatetime.h>
#include <qlineedit.h>
#include <qimage.h>

#include "libgadu.h"
#include "userlist.h"

#define __c2q(__char_pointer__) QString::fromLocal8Bit(__char_pointer__)

/*
	Zmienia sciezke relatywna do katalogu z ustawieniami gg
	na sciezke bezwzgledna uwzgledniajac zmienne srodowiskowe
	$HOME i $CONFIG_DIR
*/
QString ggPath(QString subpath);

//#define i18n(String) __c2q(gettext(String))

QString cp2unicode(unsigned char *);
QCString unicode2cp(const QString &);
QString latin2unicode(unsigned char *);
QCString unicode2latin(const QString &);
QPixmap loadIcon(const QString &filename);
QString printDateTime(const QDateTime &datetime);
QString timestamp(time_t = 0);
QDateTime currentDateTime();
QString pwHash(const QString tekst);
QString translateLanguage(const QApplication *application, const QString &locale, const bool l2n);
void openWebBrowser(const QString &link);
void escapeSpecialCharacters(QString &msg);
QString formatGGMessage(const QString &msg, int formats_length, void *formats);
QString unformatGGMessage(const QString &msg, int &formats_length, void *&formats);
QString parse(QString s, UserListElement ule, bool escape = true);

class UinsList : public QValueList<uin_t>
{
	public:
		UinsList();
		bool equals(UinsList &uins);
		void sort();
};

class ChooseDescription : public QDialog {
	Q_OBJECT
	public:
		ChooseDescription ( int nr, QWidget * parent=0, const char * name=0);

	private:
		QComboBox *desc;
		QLabel *l_yetlen;

	private slots:
		void okbtnPressed();
		void cancelbtnPressed();
		void updateYetLen(const QString&);

};

struct iconhandle {
	QString name;
	QPixmap pixmap;
};

class IconsManager {
	public:
		IconsManager();
		IconsManager(QString &dir);
		void setDirectory(QString &dir);
		QPixmap *loadIcon(QString name);
		void clear();

	private:
		QValueList<iconhandle> icons;
		QString directory;
};

extern IconsManager *icons;

/**
	Klasa reprezentujca dokument html. Przechowuje
	list elementw. Element moe by tagiem html
	albo kawakiem tekstu.
**/
class HtmlDocument
{
	private:
		struct Element
		{
			QString text;
			bool tag;
		};
		QValueList<Element> Elements;
		void escapeText(QString& text);
		void unescapeText(QString& text);
		void addElement(Element e);

	public:
		/**
			Dodaje podany tekst do listy elementw jako
			tag html.
			@param text tekst do dodania
		**/
		void addTag(const QString &text);
		/**
			Dodaje podany tekst do listy elementw jako
			zwyky tekst.
			@param text tekst do dodania
		**/
		void addText(const QString &text);
		/**
			Parsuje podany napis zgodny ze struktur html
			i otrzymane dane przypisuje do dokumentu.
			@param html napis do skonwertowania
		**/
		void parseHtml(const QString& html);
		/**
			Na podstawie zawartoci dokumentu generuje
			napis zgodny ze struktur html. Znaki specjalne
			wchodzce w skad elementw nie bdcych tagami
			html s escapowane.
		**/
		QString generateHtml();
		/**
			Zwraca ilo elementw wchodzcych w skad
			dokumentu.
		**/
		int countElements();
		/**
			Sprawdza czy element o podanym indeksie jest
			tagiem html czy zwykym tekstem.
		**/
		bool isTagElement(int index);
		/**
			Zwraca tekst elementu o podanym indeksie.
		**/
		QString elementText(int index);
		/**
			Ustawia tekst i typ elementu o podanym indeksie.
		**/			
		void setElementValue(int index,const QString& text,bool tag=false);
		/**
			Wydziela z elementu podany fragment tekstu.
			Element moe pozosta jeden albo
			ulec podzieleniu na dwa lub nawet trzy elementy.
			Funkcja zmienia warto argumentu index, aby
			indeks wydzielonego elementu by aktualny.
		**/
		void splitElement(int& index,int start,int length);
};

HtmlDocument GGMessageToHtmlDocument(const QString &msg, int formats_length, void *formats);
void HtmlDocumentToGGMessage(HtmlDocument &htmldoc, QString &msg,
	int &formats_length, void *&formats);

class ImageWidget : public QWidget
{
        Q_OBJECT

        private:
                QImage Image;

        protected:
                virtual void paintEvent(QPaintEvent *e);

        public:
		ImageWidget(QWidget *parent);
                ImageWidget(QWidget *parent,const QByteArray &image);
		void setImage(const QByteArray &image);
};

class token : public QObject {
        Q_OBJECT

        public:
                token();
                ~token();
                void getToken();

        private:
                struct gg_http *h;
                QSocketNotifier *snr;
                QSocketNotifier *snw;

                void deleteSocketNotifiers();
                void createSocketNotifiers();

        private slots:
                void socketEvent();
                void dataReceived();
                void dataSent();

        signals:
                void gotToken(struct gg_http *h);
                void tokenError();
};

class TokenDialog : public QDialog {
	Q_OBJECT

	public:
		TokenDialog(QDialog *parent = 0, const char *name = 0);
		void getToken(QString &Tokenid, QString &Tokenval);

	private slots:
		void gotTokenReceived(struct gg_http *h);
		void tokenErrorReceived();

	private:
		token Token;
		ImageWidget *tokenimage;
		QLineEdit *tokenedit;
		QString tokenid;
};

#endif
